︠b7665fc9-6863-439d-ac21-8bf4965e3221︠

def remove_squares(s):
    """"
    Removes squares from a string s under the assumption that
    the prefix s[:-1] is square free.

    Each suffix of length i, s[:-i], is compared to the second to last
    suffix of length i, s[-2*i:i], for i from 1 to len(s)/2.
    """"
    n = len(s)
    for i in range(1,n/2+1):
        if s[-i:] == s[-2*i:-i]:
            s = s[:-2*i]
            break
    return s

def meta_remove_squaresk(k):
    """
    This function returns a function that removes squares from the last
    2k characters of s.

    The returned function works just as remove_squares (above) except it
    just checks suffixes of length i from 1 to k.
    """
    def remove_squaresk( s ):
        n = len(s)
        for i in range(1,k+1):
            if s[-i:] == s[-2*i:-i]:
                s = s[:-2*i]
                break
        return s
    return remove_squaresk

def process(n=50, alphabet='abc', cleaner = remove_squares):
    """
    Simulates the random process of growing an initially empty string s over a
    given alphabet by adding n characters. At each of the n steps of the
    process a random character from the alphabet is chosen and
    it is appended. After appending the character we call a cleaner
    function on s.

    INPUT:

     - n - number of characters to append to the empty string. Default value
           is 50.
     - alphabet - Source of characters for the string. {a,b,c} is the default
           alphabet

     - cleaner - Function used to clean s. By default we remove squares from s by
                 using remove_squares.

    OUTPUT:

    - s - string that results from running the simulation.
    - max_length - maximum length of s during simulation.
    """
    s = ''
    max_length=0
    for i in xrange(n):
        c = alphabet[randint(0,len(alphabet)-1)]
        s+=c

        s = cleaner( s )
        if len(s)>max_length:
            max_length=len(s)
    return s,max_length

def expected_length( repeat, steps=100, alphabet='abc', cleaner=remove_squares ):
    """
    Runs process (defined above) for repeat many times. All other arguments are passed
    to the process function. Reports the average length of the string after running
    process for repeat times and also the length of the longest string found by process
    during the simulations.
    """
    lengths =[]
    max_length = 0
    for i in range(repeat):
        s,maxl = process(steps,alphabet, cleaner)
        lengths.append(len(s))
        if max_length < maxl:
            max_length=maxl
    return sum(lengths)/float(repeat), max_length
︡7b40b9fa-9157-46ef-a431-ae4edd63cb9f︡
︠ac1d10cf-749c-4230-a963-8c6d56684cfc︠
expected_length(1000,10000,cleaner=meta_remove_squaresk(2))
︡bc30244b-bddd-4a6e-8158-e0b040c37581︡{"stdout":"(8.51, 97)"}︡{"stdout":"\n"}︡
︠451f5a58-5e7f-4955-ad55-5596e985ad4d︠
expected_length(1000,10000,cleaner=meta_remove_squaresk(2))
︡d533bf89-b1cf-47b5-9a6d-a78c91e92e80︡{"stdout":"(8.822, 94)"}︡{"stdout":"\n"}︡
︠e149b9b4-a9dd-4107-be86-610cbedb0369︠
expected_length(1000,10000,cleaner=meta_remove_squaresk(2))
︡be8e45f9-1df0-46a6-82c8-96cc7e9a8a8e︡{"stdout":"(8.472, 90)"}︡{"stdout":"\n"}︡
︠5777be4f-1dd0-4fc8-94f2-990ecb04643d︠
expected_length(1000,10000,cleaner=meta_remove_squaresk(2))
︡4b205f10-b6ed-4078-9347-e7a1f8754dad︡{"stdout":"(8.376, 99)"}︡{"stdout":"\n"}︡
︠d7df1868-698a-4183-92b6-54cc0a696d5a︠
expected_length(1000,1000,cleaner=meta_remove_squaresk(1))
︡8733e453-ecaa-4312-a82a-b2af918aa7e8︡{"stdout":"(333.686, 442)"}︡{"stdout":"\n"}︡
︠f664da4b-9738-4d5a-8706-5e42dd608c85︠
expected_length(1000,10000)
︡8015fa14-a9d5-4543-ae46-b1f0a36d92f7︡{"stdout":"(3.074, 27)"}︡{"stdout":"\n"}︡
︠4e73ee27-9240-4267-bf4f-f7adc2fa5d64︠
expected_length(1000,10000)
︡3459355f-91dd-4b06-98d9-a1b3fb4b2f04︡{"stdout":"(3.244, 29)"}︡{"stdout":"\n"}︡
︠0f092f99-3d55-4aba-a8ce-60535b32f410︠
len(process(100000,'abcd'))
︡3d7b1964-3904-4408-a530-a8eab6ec4792︡{"stdout":"7008"}︡{"stdout":"\n"}︡
︠1cbace2d-2993-44d5-b3cf-31ebdf941b5d︠
x[-4:]
︡99e6138b-95ab-4445-bf8f-a562d329275c︡{"stdout":"'fghi'\n"}︡
︠241be384-6d63-4b50-b6ea-7b53f831e0d9︠
i=3
print x[-i:]
print x[-2*i:-i]
︡d2825710-a2ae-4e69-b365-34fbcd74f3ec︡{"stdout":"ghi\n"}︡{"stdout":"def\n"}︡
︠8334af96-1c81-4d7d-bbcf-ec4cb6daa12a︠
print x[:-2*i]
︡7611cbdf-d0a4-4c70-9b82-e5b1377b71ab︡{"stdout":"abc\n"}︡
︠fe8e0c73-b20a-4638-97d1-fa73114dcf26︠









